function  result = sirene_analyze(cvresult, precision, varargin)

% REQUIRED:
%   This functions calls the following functions that must be on the path:
%       rocanalysis.m
%
% USAGE:
%   results = sirene_analyze(cvresult)
%
% INPUT:
%   - cvresult = an array with sireneData.nsplitCv structures, each containing:
%               o score : a list of predicted scores
%               o label : a list of +1/-1 labels corresponding to the scores
%   - precision = a vector of precision levels for which one wants to
%     collect the corresponding thresholds for prediction from the SVM scores
%
% ADDITIONAL ARGUMENTS:
%   - rocCurveLength : the length of the ROC and PR (Precision/Recall)
%     curves (default : 1000)
%   - resultpath : the path where the produced files will be saved (default
%     : in current directory)
%   - plotTrue : if set to 0, allows not to plot the curves (default = 1)
%
% OUTPUT:
%   - A structure result with three components is produced
%     o result.precision : a vector of precision levels which the user wants to test
%     o result.threshold : the vector of corresponding thresholds for
%       prediction.
%     o result.recall : a vector with the corresponding levels of recall
%       obtained from the precision/recall curve
%     o result.tfstat : a structure containing the auc, aup and number of
%     targets per TF.
%   - produces ROC and PR curves (if plotTrue=1)
%   - plots the AUP for each TF as a function of the number of its known
%     regulated genes (if plotTrue=1)
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Default behaviour
rocCurveLength = 1000;
resultpath = './';
plotTrue = 1;

% Parse optional arguments
l=1;
while (l<length(varargin))
    eval(strcat(varargin{l},' = ''', varargin{l+1}, ''''));
    l = l+2;
end

disp('Now computing roc and precision/recall curves')
% recontruct label and score matrix from structure cvresult
s=0;
for ifold = 1:length(cvresult)
    s = s + size(cvresult{ifold}.label, 1);
end
label = zeros(s, size(cvresult{ifold}.label, 2));
score = zeros(s, size(cvresult{ifold}.label, 2));

k=1;
for ifold = 1:length(cvresult)
    s = size(cvresult{ifold}.label, 1);
    label(k:(k+s-1), :) = cvresult{ifold}.label;
    score(k:(k+s-1), :) = cvresult{ifold}.score;
    k = k+s;
end

% rocanalysis
r = sirene_rocanalysis(label(:),score(:),rocCurveLength);

disp('Now computing performance as a function of the number of genes regulated')

% number of genes regulated by each TF
tfstats=struct();
tfstats.nreg = sum(label>0);
tfstats.ntfs = size(label,2);

% AUC and FDR of each TF
for i=1:tfstats.ntfs
    restemp = sirene_rocanalysis(label(:,i), score(:,i), rocCurveLength);
    tfstats.auc(i) = restemp.auc;
    tfstats.aup(i) = restemp.aup;
end

if plotTrue==1
    disp('Now plotting nice curves')
    % create the path to results if it does not exist yet
    existPath = eval(strcat(['exist (''',resultpath,''')']));
    if existPath == 0
        eval(strcat(['mkdir (''',resultpath,''')']))
    end
    % Pretty-plot ROC curves
    clf
    hold on

    plot([0:1/(rocCurveLength-1):1],r.roc, 'b')

    %legend('SIRENE','Location','Southeast')
    xlabel('Ratio of false positives')
    ylabel('Ratio of true positives')
    grid on
    hold off
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'roc.eps'',''Format'',''eps'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',2)'])));
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'roc.png'',''Format'',''png'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',2)'])));

    % Pretty-plot precision/recall curves
    clf
    hold on

    plot([0:1/(rocCurveLength-1):1],r.pr, 'b')

    %legend('SIRENE','Location','Southwest')
    xlabel('Recall')
    ylabel('Precision')
    grid on
    hold off
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'prerec.eps'',''Format'',''eps'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',2)'])));
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'prerec.png'',''Format'',''png'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',2)'])));

    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    % nice plots
    clf
    semilogx(tfstats.nreg,tfstats.aup,'o')
    xlabel('Number of regulated genes')
    ylabel('AUP')
    grid on
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'aup_sirene_pertf.eps'',''Format'',''eps'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',1)'])));
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'aup_sirene_pertf.png'',''Format'',''png'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',1)'])));

    clf
    semilogx(tfstats.nreg,tfstats.auc,'o')
    xlabel('Number of regulated genes')
    ylabel('AUC')
    grid on
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'auc_sirene_pertf.eps'',''Format'',''eps'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',1)'])));
    eval(char(strcat([ 'exportfig(gcf,''',resultpath, 'auc_sirene_pertf.png'',''Format'',''png'',''FontMode'', ''fixed'', ''FontSize'', 10, ''width'' , 10 , ''height'',8,''Color'',''cmyk'',''linemode'',''fixed'',''linewidth'',1)'])));
    close

end

disp('Now computing thresholds for specified precision levels')

x=(0:1/(rocCurveLength-1):1);

ind = zeros(1, length(precision));
recall = zeros(1, length(precision));
th = zeros(1, length(precision));

if any(precision >= 1) || any(precision <= 0)
    error('Warning : precision levels must be strictly comprised between 0 and 1')
end

k = 1;
for prec = precision

    indtemp = find((r.pr) >= prec);
    if isempty(indtemp)
        disp(strcat(['Warning : level of precision ', num2str(prec), ' not reached']))
        recall(k) = NaN;
        th(k) = +inf;
        k = k+1;
    else
        ind(k) = indtemp(end);

        recall(k) = (x(ind(k))+x(ind(k)+1))/2;

        th(k) = (r.scorerecall(ind(k))+r.scorerecall(ind(k)+1))/2;
        k = k+1;

    end
end
disp(strcat('Information : Maximum level of precision reached =', num2str(max(r.pr))))

result.precision = precision;
result.threshold = th;
result.recall = recall;
result.tfstats = tfstats;

