function scores = sirene_correct(sireneData)

% USAGE:
% scores = sirene_correct(sireneData)
%
% Predicts new edges within the partial known regulation matrix 
% with SIRENE using libsvm
%
% REQUIRED:
%  - libsvm should be on the path
%
% INPUTS:
%  - sireneData : a structure containing the features, the edges and
%  various parameters, as initialized by sirene_initialize()
%
% OUTPUT:
%  a k X s matrix of scores corresponding to the edges starting from the s
%  source vertices to the k targets involved in the known matrix (ie the
%  unique list of targets indexes in sireneData.edge)


% Matrix of known edges starting from vertices in the sourcevertices list
% (one column for each source vertex)
[~, ~, indSource]=unique(sireneData.edge(:,1));
Agold = -ones(sireneData.nvertex,sireneData.nsource);
for i=1:sireneData.nedge
    Agold(sireneData.edge(i,2),indSource(i)) = 1;
end

% Main loop: compute the prediction scores for edges starting from each
% source vertex in turn
scores = zeros(sireneData.nvertex,sireneData.nsource);

% don't forget to convert sigma to gamma for the Gaussian kernel
gamma = 1/(2*sireneData.param*sireneData.param);

for isource=1:sireneData.nsource
    % build the local model for edges starting from the isource-th source
    % vertex
    labels = Agold(:,isource);
    trainpos = intersect(find(labels>0), sireneData.knownTargets);
    npos = size(trainpos,1);
    allneg = intersect(find(labels<0), sireneData.knownTargets);

    % libsvm bugs when there is no positive example
    if npos==0
        scores(:,isource)=-1;
    else

        % Make a random split of the negative examples that keep same in the
        % same split if they are in the same group
        mysplit = groupsplit(allneg,sireneData.group(allneg),sireneData.nsplitPredict);
        for isplit = 1:sireneData.nsplitPredict
            trainneg = mysplit{isplit}.train;
            testneg = mysplit{isplit}.test;
            alltrain = [trainpos;trainneg];

            % Train the SVM
            model = svmtrain(labels(alltrain),sireneData.feature(alltrain,:),strcat(['-t 2 -g ',num2str(gamma),' -c ',num2str(sireneData.C)]));

            % Make the prediction on test data
            [p,a,v] = svmpredict(labels(testneg),sireneData.feature(testneg,:),model);

            % Warning, the decision function of libsvm considers the first
            % label as the positive class...
            scores(testneg,isource) = v*labels(alltrain(1));
        end

        % Don't forget to put a good score to the positive examples!
        scores(trainpos,isource) = sireneData.defaultPositiveScore;
    end

end

scores = scores(sireneData.knownTargets,:);

