
function cvresult = sirene_cv(sireneData)
%
% cvresult = sirene_cv(sireneData)
%
% Compute the performance of SIRENE 
% Run a cross-validation to evaluate the performance of SIRENE and
% calibrate the precision as a function of SIRENE score. In each fold, some
% positive edges are labeled as negative, then SIRENE is run, and we record
% the predicted scores of all negatives.
%
% INPUT
%  - a sireneData variable
%
% OUTPUT
%  - an array with sireneData.nsplitCv structures, each containing:
%       o score : a list of predicted scores
%       o label : a list of +1/-1 labels corresponding to the scores
%

% split the positive edges into nsplitCv train/test groups, in such a way
% that two edges with target vertices in the same group are always in the
% same split.
edgeSplit = groupsplit([1:sireneData.nedge],sireneData.group(sireneData.edge(:,2)),sireneData.nsplitCv);

% creation of an object of type sireneData, restricted to genes in the
% known network
sData = sireneData;

[x y z]= unique(sireneData.edge);
ind = 1:length(x);
ind = ind(z);
edge = [ ind(1:sireneData.nedge)' , ind((sireneData.nedge+1):(2*sireneData.nedge))'];

sData.knownTargets = unique(edge(:,2));
sData.source = unique(edge(:,1));

sData.feature = sData.feature(x, :);
sData.nvertex = length(x);

sData.group = sData.group(x);
sData.groupList = unique(sData.group);
sData.ngroup = length(sData.groupList);

% gold matrix reduced to non-orphan targets
Agold = zeros(sData.nvertex,sData.nvertex);
for i=1:sireneData.nedge
    Agold(edge(i,2),edge(i,1)) = 1;
end
Agold = Agold(:, sData.source);

clear x y z ind

% preallocating
cvresult = cell(1, sireneData.nsplitCv);
% initialization
for k=1:sireneData.nsplitCv
    cvresult{k} = struct('label', [], 'score', []);
end

% cross validation loop
for ifold = 1:sireneData.nsplitCv

    disp(['Now starting iteration ',int2str(ifold),' out of ',int2str(sireneData.nsplitCv)])

    sData.edge = edge(edgeSplit{ifold}.train,:);
    sData.nedge = length(edgeSplit{ifold}.train);
    sData.knownTargets = unique(sData.edge(:,2));
    sData.toPredict = unique(edge(edgeSplit{ifold}.test,2));
    
    cvresult{ifold}.score = sirene_predict(sData); 
    cvresult{ifold}.label = Agold(sData.toPredict,:);
    
end




