function sireneData = sirene_initialize(featureFile,edgeFile,varargin)

% USAGE:
% sireneData = sirene_initialize(featureFile,edgeFile)
%
% Read data from files
%
% INPUT
%  - featureFile : a tab-delimited file containing numeric values for the
%  vertex features. The successive N rows are the features of the successive
%  N vertices. All rows should have the same number p of attributes.
%  - edgeFile : a tab-delimited file where each line contains two integers,
%  corresponding to the indices of the source and destination vertices of an
%  edge. The indices should be coherent with the ordering of rows in
%  featurefile, in particular no index should be larger than N.
%
% ADDITIONAL ARGUMENTS
%  Several other arguments can be input, by specifying the argument name
%  followed by its value:
%  - knownTargets : the non-orphan targets on which cross validation will
%  be performed, under the shape of a vector of indexes between 1 and N
%  (default : all non-orphan targets)
%  - toPredict : the targets on which prediction should be performed
%  (default : all orphan targets)
%  - kernelType : type of kernel used (default : gaussian 'rbf')
%  - geneFile : a file containing N lines with the names of the vertices
%  - groupFile : a file containing N lines, each line having an
%  integer to describe to which group the corresponding vertex
%  belongs to. Two vertices in the same group should have the same integer.
%  This information is used to make group-stratified cross-validation. For
%  example, this is useful to describe the organization of genes into
%  operons. By default, all vertices belong to different groups.
%  - sourceFile : a file containing the list of indices (one per
%  line) of the vertices that are considered as source vertices, i.e., for
%  which a prediction is needed. By default, if no file is provided, this
%  is the list of vertices with at least one outgoing edge in edgeFile.
%  - method : [simplesvm|libsvm] the method used for inference (default:
%  simplesvm)
%  - C : the 'C' regularization parameter of the SVM (default: 1000)
%  - param : the parameter of the kernel 
%            o if a gaussian kernel is chosen, this is the 'sigma'
%            parameter
%            NB : it is automatically converted to gamma if you use libsvm
%            (default: a heuristic is used to adapt it to the data, only 
%            valid when data are normalized)
%            o otherwise it should be the parameters demanded by simpleSVM
%            or libsvm depending on which method you chose
%              (default = -1, which does not make any sense)
%  - nsplitPredict : how many splits are created among the negative
%  examples to make predictions (default: 3)
%  - nsplitCv : number of folds for cross-validation (default: 3)
%  - normalizeFeatures : normalize (1) or not (0) the vertex features to
%  zero mean and unit standard deviation (default: 1).
%  - defaultPositiveScore : the score of known positive edges when edges
%  will be predicted within the known matrix with function sirene_correct() (default: 100)
%  - rocCurveLength : the length of the ROC and PR curves (default : 1000)
%  
% OUTPUT:
% - a structure variable with the following fields:
%      o feature : a N x p matrix of features
%      o edge : a r x 2 matrix of edges
%      o nedge : the number of edges
%      o nvertex : the number of vertices
%      o nfeature : the number of features
%      o knownTargets : targets used for CV
%      o kernelType : type of kernel used
%      o geneFile : the gene names file
%      o groupFile : the vertex group file name
%      o group : a N x 1 vector of group (operon) assignment
%      o groupList : the ordered list of groups
%      o ngroup : the number of groups
%      o sourceFile : the source vertex file name
%      o source : a s x 1 vector of source vertex indices
%      o nsource : the number of source vertices
%      o C : the SVM C parameter
%      o param : the kernel parameters
%      o nsplitPredict : the number of folds for prediction
%      o nsplitCv : the number of folds for cross-validation
%      o normalizeFeatures : whether or not the vertex features are
%      normalized
%      o defaultPositiveScore : the score of known positive edges 
%      o rocCurveLength : length of ROC and PR curves
%
% EXAMPLE:
% d = sirene_initialize('expression.txt','edges.txt','C',10,'groupFile','operons.txt')


% Read the vertex features
sireneData.feature = dlmread(featureFile);
[sireneData.nvertex,sireneData.nfeature] = size(sireneData.feature);

% Read the edges
sireneData.edge = dlmread(edgeFile);
sireneData.edge = sortrows(sireneData.edge);
sireneData.nedge = size(sireneData.edge,1);

% Known targets
sireneData.knownTargets = unique(sireneData.edge(:,2));
% To predict
sireneData.toPredict = setdiff(1:sireneData.nvertex, sireneData.knownTargets);

% Default values for other variables
sireneData.kernelType = 'rbf';
sireneData.geneFile = '';
sireneData.groupFile = '';
sireneData.group = [1:sireneData.nvertex]';
sireneData.groupList = [1:sireneData.nvertex]';
sireneData.ngroup = sireneData.nvertex;
sireneData.sourceFile = '';
sireneData.source = unique(sireneData.edge(:,1));
sireneData.nsource = length(sireneData.source);
sireneData.method = 'simplesvm';
sireneData.C = 1000;
sireneData.param = -1;
sireneData.nsplitPredict = 3;
sireneData.nsplitCv = 3;
sireneData.normalizeFeatures = 1;
sireneData.defaultPositiveScore = 100;
sireneData.rocCurveLength = 1000;

% Parse optional arguments
l=1;
while (l<length(varargin))
    sireneData.(varargin{l}) = varargin{l+1};
    l = l+2;
end

% Process the optional arguments
if (~isempty(sireneData.groupFile))
    sireneData.group = dlmread(sireneData.groupFile);
    sireneData.groupList = unique(sireneData.group);
    sireneData.ngroup = length(sireneData.groupList);
end

if (~isempty(sireneData.sourceFile))
    sireneData.source = dlmread(sireneData.sourceFile);
    sireneData.nsource = length(sireneData.source);
end

if (sireneData.normalizeFeatures)
    sireneData.feature = (sireneData.feature - mean(sireneData.feature,2)*ones(1,sireneData.nfeature)) ./ (std(sireneData.feature')'*ones(1,sireneData.nfeature)); %#ok<UDIM>
end

if (strcmp(sireneData.kernelType , 'rbf') && sireneData.param<0)
    sireneData.param = sqrt(2*sireneData.nfeature)/4; % a heuristic that does not make much sense if data are not normalized
end
